@@ -1,6 +1,6 @@
 /*
  * rsyncrypto - an rsync friendly encryption
- * Copyright (C) 2005,2006 Shachar Shemesh for Lingnu Open Source Consulting ltd.
+ * Copyright (C) 2005-2008 Shachar Shemesh for Lingnu Open Source Consulting ltd.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
  * The project's homepage is at http://sourceforge.net/projects/rsyncrypto
  */
 
-
+#include "precomp.h"
 #include "rsyncrypto.h"
 #include "file.h"
 #include "autodir.h"
@@ -60,8 +60,14 @@ int calc_trim( const char *path, int trim_count )
     if( path[0]=='\0' )
         throw rscerror("Cannot trim empty path");
 
-    if( trim_count==0 )
-	return 0;
+    if( trim_count==0 ) {
+        // Even if the trim is 0, we still want to seperate out the leading slashes
+        int i;
+        for( i=0; path[i]==DIRSEP_C; ++i )
+            ;
+
+	return i;
+    }
 
     do {
         if( (path[ret]==DIRSEP_C || path[ret]=='\0') && ret!=0 && path[ret-1]!=DIRSEP_C )
@@ -90,43 +96,46 @@ void filelist_encrypt( const char *src, const char *dst_dir, const char *key_dir
         srcfd=autofd(src, O_RDONLY);
     }
 
+    bool error=false;
+
     while( !srcfd.eof() ) {
         std::string srcname=srcfd.readline();
 
         if( srcname!="" ) try {
-	    // Seperate the prefix from the actual name
-	    size_t src_offset=calc_trim( srcname.c_str(), VAL(trim) );
-	    std::string src_prefix(srcname.c_str(), src_offset);
-	    srcname=std::string(srcname.c_str()+src_offset);
-	    
-	    // Perform name (de)mangling
-        std::string src=srcnameop( src_prefix.c_str(), srcname.c_str(), 0 );
-        
-        struct stat filestat=autofd::stat( src.c_str() );
-        
-        switch( filestat.st_mode&S_IFMT ) {
-        case S_IFREG:
-            {
-                std::string dstfile=dstnameop( dst_dir, srcname.c_str(), filestat.st_mode );
-                std::string keyfile=keynameop( key_dir, srcname.c_str(), filestat.st_mode );
-                
-                struct stat dststat;
-                bool found=true;
-                
-                try {
-                    dststat=autofd::lstat( dstfile.c_str() );
-                } catch( const rscerror & ) {
-                    found=false;
-                }
-                if( !EXISTS(changed) || !found ||
-                    abs(dststat.st_mtime-filestat.st_mtime)>VAL(mod_win) )
+            // Seperate the prefix from the actual name
+            size_t src_offset=calc_trim( srcname.c_str(), VAL(trim) );
+            std::string src_prefix(srcname.c_str(), src_offset);
+            srcname=std::string(srcname.c_str()+src_offset);
+
+            // Perform name (de)mangling
+            std::string src=srcnameop( src_prefix.c_str(), srcname.c_str(), 0 );
+
+            struct stat filestat=autofd::stat( src.c_str() );
+
+            switch( filestat.st_mode&S_IFMT ) {
+            case S_IFREG:
                 {
-                    if( VERBOSE(1) )
-                        std::cerr<<opname<<" file: "<<srcname<<std::endl;
-                    
-                    op( src.c_str(), dstfile.c_str(), keyfile.c_str(), rsa_key, &filestat );
-                } else if( VERBOSE(1) ) {
-                    std::cerr<<opname<<": skipped unchanged file: "<<srcname<<std::endl;
+                    std::string dstfile=dstnameop( dst_dir, srcname.c_str(), filestat.st_mode );
+                    std::string keyfile=keynameop( key_dir, srcname.c_str(), filestat.st_mode );
+
+                    struct stat dststat;
+                    bool found=true;
+
+                    try {
+                        dststat=autofd::lstat( dstfile.c_str() );
+                    } catch( const rscerror & ) {
+                        found=false;
+                    }
+                    if( !EXISTS(changed) || !found ||
+                            abs(dststat.st_mtime-filestat.st_mtime)>VAL(mod_win) )
+                    {
+                        // Report to stderr the operation
+                        if( VERBOSE(1) )
+                            std::cerr<<opname<<" file: "<<srcname<<std::endl;
+
+                        op( src.c_str(), dstfile.c_str(), keyfile.c_str(), rsa_key, &filestat );
+                    } else if( VERBOSE(1) ) {
+                        std::cerr<<opname<<": skipped unchanged file: "<<srcname<<std::endl;
                 }
             }
             break;
@@ -146,10 +155,17 @@ void filelist_encrypt( const char *src, const char *dst_dir, const char *key_dir
             std::cerr<<"Unsupported file type. Skipping "<<src<<std::endl;
             break;
         }
+        } catch( const delayed_error &err ) {
+            error=true;
         } catch( const rscerror &err ) {
             std::cerr<<"Error in encryption of "<<srcname<<": "<<err.error()<<std::endl;
+            error=true;
         }
     }
+
+    if( error ) {
+        throw delayed_error();
+    }
 }
 
 static void recurse_dir_enc( const char *src_dir, const char *dst_dir, const char *key_dir, RSA *rsa_key,
@@ -159,60 +175,75 @@ static void recurse_dir_enc( const char *src_dir, const char *dst_dir, const cha
     autodir dir(src_dir);
 
     struct dirent *ent;
+    bool error=false;
+
     while( (ent=dir.read())!=NULL ) {
-        std::string src_filename(autofd::combine_paths(src_dir, ent->d_name));
-        
-        struct stat status, dststat;
-        status=autofd::lstat( src_filename.c_str() );
-        std::string dst_filename(dstname(dst_dir, src_filename.c_str()+src_offset, status.st_mode));
-        std::string key_filename(keyname(key_dir, src_filename.c_str()+src_offset, status.st_mode));
-        
-        if( dst_filename.length()>0 ) {
-            switch( status.st_mode & S_IFMT ) {
-            case S_IFREG:
-                // Regular file
-                {
-                    bool statsuccess=true;
-                    try {
-                        dststat=autofd::lstat( dst_filename.c_str() );
-                    } catch( const rscerror & ) {
-                        statsuccess=false;
-                    }
-                    if( !EXISTS(changed) || !statsuccess || abs(dststat.st_mtime-status.st_mtime)>VAL(mod_win) ) {
-                        if( VERBOSE(1) && opname!=NULL )
-                            std::cerr<<opname<<" "<<src_filename<<std::endl;
-                        try {
-                            op( src_filename.c_str(), dst_filename.c_str(), key_filename.c_str(),
-                                rsa_key, &status );
-                        } catch( const rscerror &err ) {
-                            std::cerr<<opname<<" "<<dst_filename<<" error: "<<err.error()<<std::endl;
+        try {
+            std::string src_filename(autofd::combine_paths(src_dir, ent->d_name));
+
+            struct stat status, dststat;
+            status=autofd::lstat( src_filename.c_str() );
+            std::string dst_filename(dstname(dst_dir, src_filename.c_str()+src_offset, status.st_mode));
+            std::string key_filename(keyname(key_dir, src_filename.c_str()+src_offset, status.st_mode));
+
+            if( dst_filename.length()>0 ) {
+                switch( status.st_mode & S_IFMT ) {
+                    case S_IFREG:
+                        // Regular file
+                        {
+                            bool statsuccess=true;
+                            try {
+                                dststat=autofd::lstat( dst_filename.c_str() );
+                            } catch( const rscerror & ) {
+                                statsuccess=false;
+                            }
+                            if( !EXISTS(changed) || !statsuccess || abs(dststat.st_mtime-status.st_mtime)>VAL(mod_win) ) {
+                                // Report to stderr (if applicable) the operation
+                                if( VERBOSE(1) && opname!=NULL )
+                                    std::cerr<<opname<<" "<<src_filename<<std::endl;
+
+                                try {
+                                    op( src_filename.c_str(), dst_filename.c_str(), key_filename.c_str(),
+                                            rsa_key, &status );
+                                } catch( const rscerror &err ) {
+                                    std::cerr<<opname<<" "<<dst_filename<<" error: "<<err.error()<<std::endl;
+                                }
+                            } else if( VERBOSE(2) && opname!=NULL ) {
+                                std::cerr<<"Skipping unchanged file "<<src_filename<<std::endl;
+                            }
                         }
-                    } else if( VERBOSE(2) && opname!=NULL ) {
-                        std::cerr<<"Skipping unchanged file "<<src_filename<<std::endl;
-                    }
+                        break;
+                    case S_IFDIR:
+                        // Directory
+                        if( strcmp(ent->d_name,".")!=0 && strcmp(ent->d_name,"..")!=0 ) {
+                            if( !op_handle_dir ) {
+                                recurse_dir_enc( src_filename.c_str(), dst_dir, key_dir, rsa_key, op,
+                                        src_offset, op_handle_dir, opname, dstname, keyname );
+                            } else {
+                                recurse_dir_enc( src_filename.c_str(), dst_dir, key_dir, rsa_key, op,
+                                        src_offset, op_handle_dir, opname, dstname, keyname );
+                                op( src_filename.c_str(), dst_filename.c_str(), key_filename.c_str(),
+                                        rsa_key, &status );
+                            }
+                        }
+                        break;
+                    default:
+                        // Unhandled type
+                        std::cerr<<"Skipping unhandled file type: "<<src_filename<<std::endl;
+                        break;
                 }
-                break;
-            case S_IFDIR:
-                // Directory
-                if( strcmp(ent->d_name,".")!=0 && strcmp(ent->d_name,"..")!=0 ) {
-                    if( !op_handle_dir ) {
-                        recurse_dir_enc( src_filename.c_str(), dst_dir, key_dir, rsa_key, op,
-                            src_offset, op_handle_dir, opname, dstname, keyname );
-                    } else {
-                        recurse_dir_enc( src_filename.c_str(), dst_dir, key_dir, rsa_key, op,
-                            src_offset, op_handle_dir, opname, dstname, keyname );
-                        op( src_filename.c_str(), dst_filename.c_str(), key_filename.c_str(),
-                            rsa_key, &status );
-                    }
-                }
-                break;
-            default:
-                // Unhandled type
-                std::cerr<<"Skipping unhandled file type: "<<src_filename<<std::endl;
-                break;
             }
+        } catch( const delayed_error &err ) {
+            error=true;
+        } catch( const rscerror &err ) {
+            std::cerr<<err.error()<<std::endl;
+            error=true;
         }
     }
+
+    if( error ) {
+        throw delayed_error();
+    }
 }
 
 static void file_delete( const char *source_file, const char *dst_file, const char *key_file,
@@ -237,17 +268,31 @@ static void file_delete( const char *source_file, const char *dst_file, const ch
             switch( status.st_mode & S_IFMT ) {
             case S_IFDIR:
                 // Need to erase directory
-                if( VERBOSE(1) )
-                    std::cerr<<"Delete dirs "<<dst_file<<", "<< key_file<<std::endl;
+                if( VERBOSE(1) ) {
+                    if( EXISTS(delkey) )
+                        std::cerr<<"Delete dirs "<<source_file<<", "<<key_file<<std::endl;
+                    else
+                        std::cerr<<"Delete dir "<<source_file<<std::endl;
+                }
+
+                // If an execution log is active - report the operation
+                if( changes_log.get()!=NULL ) {
+                    (*changes_log.get())<<source_file<<std::endl;
+                }
+
                 autofd::rmdir( source_file );
-                autofd::rmdir( key_file );
+                if( EXISTS(delkey) )
+                    autofd::rmdir( key_file );
                 break;
             case S_IFREG:
 #if defined S_IFLNK
             case S_IFLNK:
 #endif
                 if( VERBOSE(1) )
-                    std::cout<<"Delete "<<source_file<<std::endl;
+                    std::cerr<<"Delete "<<source_file<<std::endl;
+                // If an execution log is active - report the operation
+                if( changes_log.get()!=NULL )
+                    (*changes_log.get())<<source_file<<std::endl;
                 if( unlink( source_file )!=0 )
                     throw rscerror("Erasing file", errno, source_file );
                 if( EXISTS(delkey) ) {
@@ -300,6 +345,11 @@ void dir_encrypt( const char *src_dir, const char *dst_dir, const char *key_dir,
 void file_encrypt( const char *source_file, const char *dst_file, const char *key_file,
         RSA *rsa_key, const struct stat *stat )
 {
+    // If an execution log is active - report the operation
+    if( changes_log.get()!=NULL ) {
+        (*changes_log.get())<<dst_file<<std::endl;
+    }
+
     std::auto_ptr<key> head;
     autofd headfd;
 
@@ -315,10 +365,10 @@ void file_encrypt( const char *source_file, const char *dst_file, const char *ke
             autommap headmap( headfd, PROT_READ );
             head=std::auto_ptr<key>(key::read_key( static_cast<unsigned char *>(headmap.get()) ));
 
-            if( EXISTS(fr) && ( head->get_sum_span()!=static_cast<uint32_t>(VAL(rollwin)) ||
+            if( ( EXISTS(fr) && ( head->get_sum_span()!=static_cast<uint32_t>(VAL(rollwin)) ||
                         head->get_sum_mod()!=static_cast<uint32_t>(VAL(rollsens)) ||
-                        head->get_sum_min_dist()!=static_cast<uint32_t>(VAL(rollmin))) ||
-                    EXISTS(fk) && head->get_key_size()!=static_cast<uint32_t>(VAL(keysize)) ) {
+                        head->get_sum_min_dist()!=static_cast<uint32_t>(VAL(rollmin))) ) ||
+                    ( EXISTS(fk) && head->get_key_size()!=static_cast<uint32_t>(VAL(keysize)) ) ) {
                 headfd.clear();
             }
                 
