@@ -1,6 +1,6 @@
 /*
  * rsyncrypto - an rsync friendly encryption
- * Copyright (C) 2005 Shachar Shemesh for Lingnu Open Source Consulting ltd.
+ * Copyright (C) 2005-2008 Shachar Shemesh for Lingnu Open Source Consulting ltd.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,13 +28,15 @@
  * The project's homepage is at http://sourceforge.net/projects/rsyncrypto
  */
 
-
+#include "precomp.h"
 #include "rsyncrypto.h"
 #include "file.h"
 #include "crypto.h"
 #include "argtable2.h"
 #include "filemap.h"
 
+std::auto_ptr<std::ostream> changes_log;
+
 void version()
 {
     printf("%s by Shachar Shemesh\n", PACKAGE_STRING);
@@ -55,9 +57,9 @@ startup_options options;
 void parse_cmdline( int argc, char *argv[] )
 {
     int nerrors=arg_parse( argc, argv, options.argtable );
-    if( nerrors!=0 ) {
-	std::cerr<<"Incorrect arguments"<<std::endl;
-	usage();
+	if( nerrors!=0 ) {
+		std::cerr<<"Incorrect arguments"<<std::endl;
+		usage();
     }
 
     if( EXISTS(trim) && !EXISTS(recurse) && !EXISTS(filelist) )
@@ -73,12 +75,15 @@ void parse_cmdline( int argc, char *argv[] )
             throw rscerror("Cannot specify key size for decryption");
         if( EXISTS(fr) || EXISTS(fk) )
             throw rscerror("\"force\" options incompatible with -d option");
-        if( strcmp(FILENAME(src), "-")==0 ) {
+        if( strcmp(FILENAME(src), "-")==0 && !EXISTS(filelist) ) {
             // Plaintext file is standard input/output
             if( !EXISTS(noarch) ) {
                 throw rscerror("Must use \"--no-archive-mode\" if plaintext file is stdin");
             }
         }
+        if( EXISTS(export_changes) ) {
+            throw rscerror("Log export not supported in decrypt mode");
+        }
     }
 
     // Apply default values
@@ -109,7 +114,7 @@ int main( int argc, char *argv[] )
 
 #if HAVE_NOATIME
         // Sometimes we can always use O_NOATIME without a problem
-        if( VAL(noatime)==1 && geteuid==0 ) {
+        if( VAL(noatime)==1 && geteuid()==0 ) {
             // We are root - O_NOATIME will succeed anyways
             VAL(noatime)=2;
         }
@@ -124,6 +129,10 @@ int main( int argc, char *argv[] )
             }
         }
 
+        if( EXISTS(export_changes) ) {
+            changes_log=std::auto_ptr<std::ofstream>(new std::ofstream(FILENAME(export_changes), std::ofstream::trunc));
+        }
+
         const char *opname=NULL;
 	encryptfunc op;
 	namefunc srcnameop=name_concat, dstnameop=name_concat, keynameop=name_concat;
