@@ -1,100 +1,7 @@
 #ifndef RSYNCRYPTO_H
 #define RSYNCRYPTO_H
 
-#ifndef _WIN32
-#include "config.h"
-#define DIRSEP_C '/'
-#define DIRSEP_S "/"
-#else
-#define PACKAGE_NAME "rsyncrypto"
-#define PACKAGE_STRING "rsyncrypto 1.06"
-#define DIRSEP_C '\\'
-#define DIRSEP_S "\\"
-#pragma warning ( disable : 4786 )
-
-typedef __int64 _off_t;
-typedef _off_t off_t;
-#define _OFF_T_DEFINED
-
-#endif
-
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-
-#if defined(__unix__)
-#include <sys/mman.h>
-#include <sys/wait.h>
-#include <sys/time.h>
-#include <netinet/in.h>
-#include <unistd.h>
-#include <utime.h>
-#include <dirent.h>
-#elif defined(_WIN32)
-#define STRICT
-#include <windows.h>
-#include <sys/utime.h>
-#else
-#error Unsupported platform
-#endif
-
-#include <stdlib.h>
-#include <stdarg.h>
-
-#include <assert.h>
-#include <string.h>
-#if HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#include <stdlib.h>
-
-#include <openssl/rand.h>
-#include <openssl/bio.h>
-#include <openssl/pem.h>
-#include <openssl/err.h>
-
-#include <memory>
-#include <iostream>
-#include <string>
-#include <set>
-
-#if HAVE_EXT_HASH_MAP && 0
-#include <ext/hash_map>
-#define HASH_MAP 1
-#else
-#include <map>
-#endif
-
-#include <argtable2.h>
-
-class rscerror {
-    std::string msg;
-    std::string sysmsg;
-    std::string param;
-    int errnum;
-public:
-    explicit rscerror( const char *msg_p ) : msg(msg_p)
-    {
-    }
-    explicit rscerror( const char *msg_p, int error, const char *param_p="" ) : msg(msg_p),
-                                                                                sysmsg(strerror(error)),
-                                                                                param(param_p),
-                                                                                errnum(error)
-    {
-    }
-
-    std::string error() const {
-        std::string ret(msg);
-        if( param.length()!=0 )
-            ret+="("+param+")";
-        ret+=": "+sysmsg;
-
-        return ret;
-    }
-    int errornum() const {
-        return errnum;
-    }
-};
+#include "rcserror.h"
 
 struct startup_options {
     struct arg_lit *help, *del, *delkey, *filelist, *fr, *fk, *noarch, *version;
@@ -102,11 +9,11 @@ struct startup_options {
     struct arg_int *keysize, *rollwin, *rollmin, *rollsens, *trim, *nenest;
     struct arg_int *noatime, *mod_win;
     struct arg_file *gzip;
-    struct arg_file *src, *dst, *key, *master, *nameenc;
+    struct arg_file *src, *dst, *key, *master, *nameenc, *export_changes;
     struct arg_rem *rem1;
     struct arg_end *end;
 
-    void *argtable[27
+    void *argtable[28
 #if HAVE_NOATIME
         +1
 #endif
@@ -129,6 +36,7 @@ struct startup_options {
                 "<src> <dst> and <keys> are directory names, and are processed recursively");
         argtable[i++]=changed=arg_lit0( "c", "changed", "Only encrypt changed files. Requires -r or --filelist");
         argtable[i++]=mod_win=arg_int0( NULL, "modify-window", "<n>", "compare mod-times with reduced accuracy" );
+        argtable[i++]=export_changes=arg_file0( NULL, "export-changes", "log_file", "Write list of affected files to a log file" );
         argtable[i++]=nameenc=arg_file0( "n", "name-encrypt", "translation_file", "Encrypt file names");
         argtable[i++]=nenest=arg_int0( NULL, "ne-nesting", "<n>", "set the hash directory tree depth when encrypting file names" );
         argtable[i++]=trim=arg_int0( NULL, "trim", "<n>",
@@ -184,14 +92,11 @@ extern startup_options options;
 #define ARG(arg) (*(options.arg))
 #define VERBOSE(val) (ARG(verbosity).count>=(val))
 extern std::ostream *report0, *report1, *report2, *report3;
+extern std::auto_ptr<std::ostream> changes_log;
 
 #define EXCEPT_CLASS rscerror
 
-#include "autoarray.h"
-#if defined(__unix__)
-#include "autofd.h"
-#include "autommap.h"
-#elif defined(_WIN32)
+#if defined(_WIN32)
 
 static inline void ODS(const char *format, ... )
 {
@@ -207,10 +112,6 @@ static inline void ODS(const char *format, ... )
 }
 
 #include "win32/types.h"
-#include "win32/autofd.h"
-#include "win32/autommap.h"
-#else
-#error Unsupported platform
 #endif
 
 #endif // RSYNCRYPTO_H
