@@ -1,6 +1,6 @@
 /*
  * rsyncrypto - an rsync friendly encryption
- * Copyright (C) 2005 Shachar Shemesh for Lingnu Open Source Consulting ltd.
+ * Copyright (C) 2005-2008 Shachar Shemesh for Lingnu Open Source Consulting ltd.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
  * The project's homepage is at http://sourceforge.net/projects/rsyncrypto
  */
 
-#include "rsyncrypto.h"
+#include "../precomp.h"
 
 int Error2errno( DWORD Error ) {
 	switch( Error ) {
@@ -37,6 +37,8 @@ int Error2errno( DWORD Error ) {
 	case ERROR_FILE_NOT_FOUND:
 	case ERROR_PATH_NOT_FOUND:
 		return ENOENT;
+	case ERROR_INVALID_DRIVE:
+		return ENODEV;
 	default:
 		return Error;
 	}
